@@ -7,8 +7,13 @@ API focusing on the core use case and with automatic variable escaping< built in
 to help prevent XSS attacks. The new API supports all the features of the old
 one but is easier to understand.
 
+There is no support for default values. To handle default values you should pass
+the values you want to use as defaults as the argument to Form() for the case
+where the defaults should be displayed.
 """
 
+#? Maybe use the terminology of form template for forms?
+
 import warnings
 from formbuild import helpers
 import webhelpers.html.tags
@@ -56,6 +61,18 @@ class LayoutHelpers:
         """
         return literal('<tr><td></td><td colspan="2">')+literal('').join(fields)+literal('</td></tr>')
 
+    def hidden_fields(self, *fields):
+        output = [literal('<tr><td></td><td colspan="2">')]
+        output.append(literal('</td></tr>'))
+        for field in fields:
+            output.append(webhelpers.html.tags.hidden(field, value=self.values[field]))
+        return literal('').join(output)
+
+    def title(self, content, escape=False):
+        if escape:
+            content = literal(content)
+        return literal('<tr><td colspan="3">')+content+literal('</td></tr>')
+
 class FieldHelper:
     def field(
         self,
@@ -67,6 +84,8 @@ class FieldHelper:
         help='',
         error='',
         name=None,
+        field_pre='',
+        field_extra='',
     ):
         """\
 	Generate a field row in the form but use the dictionary of arguments
@@ -78,10 +97,11 @@ class FieldHelper:
 
         1. The value corresponding to a ``value`` key in the dictionary passed as the ``field`` argument.
         2. A suitable value passed as an argument to the constructor as the ``defualts`` argument
-        3. The value corresponding to a ``default`` key in the dictionary passed as the ``field`` argument.
         For choosing error messages, the ``error`` argument passed as an argument to this method is used if it exists, otherwise any error message passed to the class itself is used. 
         """
         if isinstance(field, dict):
+            if field.has_key('default'):
+                raise Exception("FormBuild no longer supports direct defaults, set some values instead")
             for arg in ['name', 'type']:
                 if not field.has_key(arg):
                     raise Exception('No %r key passed in the "field" argument'%arg)
@@ -90,15 +110,41 @@ class FieldHelper:
             if not hasattr(self, type):
                 raise Exception('No such field %r associated with this form'%type)
             name = field['name']
-            value = None
-            if field.has_key('value'):
-                value = field['value']
-            elif self.values.has_key(name):
-                value = self.values.get(name)
-            elif field.has_key('default'):
-                value = field['default']
-                del field['default']
-            field['value'] = value
+            if type in ['checkbox_group', 'radio_group', 'combo', 'dropdown']:
+                selected_values = []
+                if field.has_key('value'):
+                    raise Exception("%s fields take a 'selected_values' argument, not a 'value' argument"%type)
+                if field.has_key('selected_values'):
+                    selected_values = field['selected_values']
+                elif self.values.has_key(name):
+                    selected_values = self.values.get(name)
+                field['selected_values'] = selected_values
+                if field.has_key('options'):
+                    field['options'] = self.options[name]
+                elif self.options.has_key(name):
+                    field['options'] = self.options[name]
+                else:
+                    require_options = field.get('require_options')
+                    if require_options is None:
+                        # Just don't set any options
+                        log.warning('No options were specified for the checkbox group %r', name)
+                        del field['require_options']
+                        field['options'] = []
+                    # Use == in case someone sets to 0
+                    elif require_options == True:
+                        raise Exception('No options were specified for the checkbox group %r' % name)
+                    else:
+                        field['options'] = []
+                        del field['require_options']
+                if field.has_key('require_options'):
+                    del field['require_options']
+            else:
+                value = None
+                if field.has_key('value'):
+                    value = field['value']
+                elif self.values.has_key(name):
+                    value = self.values.get(name)
+                field['value'] = value
             field_html = getattr(self, type)(**field)
             error_html = error or self.errors.get(name)
         else:
@@ -108,23 +154,31 @@ class FieldHelper:
             error_html = error or self.errors.get(name)
         return helpers.field(
             label=label, 
-            field=field_html,
+            field=field_html+field_extra,
             required=required, 
             label_desc=label_desc, 
             field_desc=field_desc,
             help=help,
-            error=error_html
+            error=error_html,
+            field_pre=field_pre,
         )
         
 class BaseForm:
-    def __init__(self, values=None, errors=None, state=None):
-        self.values = values
-        self.errors = errors
-        self.state = state 
+    def __init__(self, values=None, errors=None, state=None, options=None):
         if values is None:
             self.values = {}
+        else:
+            self.values = values
         if errors is None:
             self.errors = {}
+        else:
+            self.errors = errors
+        if options is None:
+            self.options = {}
+        else:
+            self.options = options
+        self.state = state
+   
 
 class SingleValueFields:
     """\
@@ -155,14 +209,23 @@ class SingleValueFields:
         """Creates a check box."""
         if value is None:
             value = self.values.get(name)
-        if value == 'on':
+        elif value == self.values.get(name) or self.values.get(name) == True:
             checked=True
         return webhelpers.html.tags.checkbox(name, value=value, checked=checked, id=id, **options)
     
     def radio(self, name, value, checked=False, id=None, **options):
         """Creates a radio button."""
         return webhelpers.html.tags.radio(name, value, checked, id=id, **options)
-    
+   
+    def image_button(self, name='commit', value="", id=None, **options):
+        """\
+        Creates an image button with the text ``<tt>value</tt>`` as the 
+        caption.
+        """
+        if value is None:
+            value = self.values.get(name)
+        return image_button(value=value, name=name, id=id, **options)
+
     def submit(self, name='commit', value="", id=None, **options):
         """\
         Creates a submit button with the text ``<tt>value</tt>`` as the 
@@ -175,7 +238,9 @@ class SingleValueFields:
     def dropdown(self, name, options, value=None, id=None, **attrs):
         if value is None:
             value = self.values.get(name)
-        return webhelpers.html.tags.select(name, selected_values=[value], options=options, id=id, **attrs)
+        if value is None and not attrs.has_key('selected_values'):
+            attrs['selected_values']=[value]
+        return webhelpers.html.tags.select(name, options=options, id=id, **attrs)
         
     def text(self, name, value=None, id=None, **options):
         if value is None:
@@ -186,15 +251,17 @@ class SingleValueFields:
         if value is None:
             value = self.values.get(name)
         return webhelpers.html.tags.hidden(name, value, id=id, **options)
-    
+   
     def radio_group(self, name, options, value=None, align='horiz', cols=4):
         if value is None:
             value = self.values.get(name)
-        if value is None:
-            selected_values = []
-        else:
-            selected_values = [value]
-        return helpers.radio_group(name, value=selected_values, options=options, align=align, cols=cols)
+        #if value is None:
+        #    selected_values = []
+        #else:
+        #    selected_values = [value]
+        #return helpers.radio_group(name, value=selected_values, options=options, align=align, cols=cols)
+        return helpers.radio_group(name, value=value, options=options, align=align, cols=cols)
+
     def static(self, name, value=None):
         """
         Return the static value instead of an HTML field.
@@ -250,42 +317,41 @@ class SingleValueFields:
         )
         return select(name, option_tags, id=id, **options)
 
-    def options(self, options, selected_values):
-        raise Exception('options() has been removed because dropdown() now generates the values safely itself')
-        warnings.warn(
-            "options() is deprecated, options are generated automatically now and don't need to be passed to other methods explicitly",
-            DeprecationWarning, stacklevel=2
-        )
-        o = ''
-        for op in options:
-            o += self.option(op, selected_values)+'\n'
-        return o
-
 class MultipleValueFields:
     """\
     These are methods which represent helpers which take multiple value arguments. When called
     directly they will take the ``value`` argument passed to them or the default from the Form
     class constructor.
     """
-    def combo(self, name, options, selected_values=None, id=None, size=4, **attrs):
+    def combo(self, name, options=None, selected_values=None, id=None, size=4, **attrs):
         if selected_values is None:
             selected_values = self.values.get(name)
+        if options is None:
+            options = self.options.get(name)
         return webhelpers.html.tags.select(name, selected_values=selected_values, options=options, size=size, multiple=True, id=id, **attrs)
 
-    def checkbox_group(self, name, options, selected_values=None, align='horiz', cols=4):
+    def checkbox_group(self, name, options=None, selected_values=None, align='horiz', cols=4, sub_name=None):
         if selected_values is None:
-            selected_values = self.values.get(name)
-        return checkbox_group(name, selected_values=selected_values, options=options, align=align, cols=cols)
+            if sub_name:
+                selected_values = []
+                for k, v in self.values.items():
+                    if k.startswith(name+'.') or k.startswith(name+'['):
+                        selected_values.append(v)
+            else:
+                selected_values = self.values.get(name)
+        if options is None:
+            options = self.options.get(name)
+        return checkbox_group(name, selected_values=selected_values, options=options, align=align, cols=cols, sub_name=sub_name)
 
-    def radio_group(self, name, options, selected_values=None, align='horiz', cols=4):
+    def radio_group(self, name, options, selected_values=None, align='horiz', cols=4, sub_name=None):
         if selected_values is None:
             selected_values = self.values.get(name)
-        return radio_group(name, selected_values=selected_values, options=options, align=align, cols=cols)
+        return radio_group(name, selected_values=selected_values, options=options, align=align, cols=cols, sub_name=sub_name)
 
 class Form(BaseForm, FieldHelper, LayoutHelpers, SingleValueFields, MultipleValueFields):
-    def __init__(self, values=None, errors=None, state=None, table_class='formbuild'):
-        BaseForm.__init__(self, values, errors, state)
-        LayoutHelpers.__init__(self, table_class)
+    def __init__(self, values=None, errors=None, state=None, table_class='formbuild', options=None):
+        BaseForm.__init__(self, values=values, errors=errors, state=state, options=options)
+        LayoutHelpers.__init__(self, table_class=table_class)
 
 #def handle(schema, template, form=None, data=None, params=None, context=None, render_response=None, fragment=False):
 #    if data == None:
@@ -327,9 +393,25 @@ class Form(BaseForm, FieldHelper, LayoutHelpers, SingleValueFields, MultipleValu
 #        return results, errors, render_response(template, fragment=fragment)
 #    return results, errors, ''
 
-def errors_to_dict(exception):
+class ValidationException(Exception):
+    pass
+
+def errors_to_dict(exception, allow_empty_keys=False):
     log.info('Converting errors %r', exception)
-    return exception.unpack_errors() or {}
+    import formencode.variabledecode
+    errors = exception.unpack_errors() or {}
+    if not allow_empty_keys:
+        if not isinstance(errors, dict):
+            raise ValidationException(errors)
+        for k, v in errors.items():
+            if not k:
+                raise ValidationException(v)
+    return formencode.variabledecode.variable_encode(errors)
+
+def values_to_dict(result):
+    log.info('Converting values %r', result)
+    import formencode.variabledecode
+    return formencode.variabledecode.variable_encode(result)
 
 def params_to_dict(params):
     result = {}
@@ -339,19 +421,26 @@ def params_to_dict(params):
             result[k] = v[0]
         else:
             result[k] = v
-    return dict(result)
+    return result
 
 class FormEncodeState(object):
     pass
 
 class ValidationState(object):
+    """
+    Wraps a PowerPack state so that attribute FormEncode needs can still be 
+    set. Also provides a .validation_state attribute for setting per 
+    instance attributes on.
+    """
+
     def __init__(self, state=None, key='validation'):
         if state is not None:
             self.__dict__['_state'] = state
         self.__dict__['_formencode_state'] = FormEncodeState()
+        self.__dict__['validation_state'] = FormEncodeState()
 
     def __setattr__(self, name, value):
-        if name in ['full_dict', 'key']:
+        if name in ['full_list', 'full_dict', 'key', 'index', 'validation_state']:
             setattr(self.__dict__['_formencode_state'], name, value)
         else:
             setattr(self.__dict__['_state'], name, value)
